@@ -7,6 +7,8 @@
 #include <algorithm>
 #include <stdlib.h>
 #include <string.h>
+#include <iostream>
+#include <limits>
 
 #include "Matrix.h"
 #include "DenseMatrix.h"
@@ -26,7 +28,6 @@ namespace blas{
   extern "C"{
 
   //---- matrix matrix multiply ----//
-  
   void dgemm_(char *transa, char *transb, int *m, int *n, int *k,
       double *alpha, double *A, int *lda, double *B, int *ldb, double *beta,
       double *C, int *ldc);
@@ -39,7 +40,6 @@ namespace blas{
 
 
   //---- matrix vector multiply ----//
-  
   void dgemv_(char *trans, int *m, int *n, double *alpha, double *A, int *lda, 
       double *x, int *incx, double *beta, double *y, int *incy);
     
@@ -51,11 +51,18 @@ namespace blas{
 
 
   //---- dot product ----/
-  
   double ddot_(int *m, double *x, int *incx, double *y, int *incy);
 
   float sdot_(int *m, float *x, int *incx, float *y, int *incy);
 
+
+  //---- QR decomposition ----//
+  void dgeqrf_(int *m, int *n, double *a, int *lda, double *tau, double *work, int *lwork, int *info);
+  void sgeqrf_(int *m, int *n, float *a, int *lda, float *tau, float *work, int *lwork, int *info);
+
+  void dorgqr_(int *m, int *n, int *k, double *a, int *lda, double *tau, double *work, int *lwork, int *info);
+  void sorgqr_(int *m, int *n, int *k, float *a, int *lda, float *tau, float *work, int *lwork, int *info);
+
   //---- LU decvomposition ----//
   void dgetrf_(int * m, int *n, double *a, int *lda, int *ipiv, int *info);
 
@@ -75,13 +82,11 @@ namespace blas{
 
 
   //---- inverse symmetric positive definite matrix ----/
-  
   void dpotri_(char *uplo, int *n, double *A, int *lda, int *info);
   
   void spotri_(char *uplo, int *n, float *A, int *lda, int *info);
 
   //---- Solve SPD linear equations ----//
-
   void dposv_(char *uplo, int *n, int *nrhs, double *A, int *lda, double *B,
       int *ldb, int *info );
 
@@ -99,7 +104,6 @@ namespace blas{
       int *iwork, int *info );
 
   //---- Solve linear equations ----/
-  
   void dgesv_( int *n, int *nrhs, double *A, int *lda, int *ipiv, double *b, int
       *ldb, int *info );
 
@@ -418,6 +422,69 @@ class Linalg{
   };
 
 
+
+  //---- QRfactorization ---//
+  static DenseMatrix<TPrecision> QR(DenseMatrix<TPrecision> &a){
+    DenseMatrix<TPrecision> q = Copy(a);
+    Linalg<TPrecision>::QR_inplace(q);
+    return q;    
+  };
+
+  static void QR_inplace(DenseMatrix<TPrecision> &q){
+    int n = q.N();
+    int m = q.M();
+    int lda = m;
+    int info = 0;
+    TPrecision *work = new TPrecision[1];
+    int lwork = -1;
+    TPrecision *tau = new TPrecision[n];
+
+    //workspace query
+    if(isDoubleTPrecision()){
+       blas::dgeqrf_(&m, &n, (double*) q.data(), &lda, (double*) tau, (double*) work, &lwork, &info);
+    }
+    else{ 
+       blas::sgeqrf_(&m, &n, (float*) q.data(), &lda, (float*) tau, (float*) work, &lwork, &info);
+    }
+    if(info !=0 ){
+      std::cerr << "QRF workspace error: " << info << std::endl;
+    }
+
+    lwork = work[0];
+    delete work;
+    work = new TPrecision[lwork];
+
+    //qr
+    if(isDoubleTPrecision()){
+      blas::dgeqrf_(&m, &n, (double*) q.data(), &lda, (double*) tau, (double*) work, &lwork, &info);
+      if(info !=0 ){
+        std::cerr << "QRF error: " << info << std::endl;
+      }
+
+      blas::dorgqr_(&m, &n, &n, (double*) q.data(), &lda, (double*) tau, (double*) work, &lwork, &info);
+      if(info !=0 ){
+        std::cerr << "QR error: " << info << std::endl;
+      }
+    }
+    else{
+      blas::sgeqrf_(&m, &n, (float*) q.data(), &lda, (float*) tau, (float*) work, &lwork, &info);
+      if(info !=0 ){
+        std::cerr << "QRF error: " << info << std::endl;
+      }
+
+      blas::sorgqr_(&m, &n, &n, (float*) q.data(), &lda, (float*) tau, (float*) work, &lwork, &info);
+      if(info !=0 ){
+        std::cerr << "QR error: " << info << std::endl;
+      }
+    }
+
+
+    delete work;
+    delete tau;
+  };
+
+
+
   //---- Choelsky factorization ---//
   static DenseMatrix<TPrecision> Cholesky(DenseMatrix<TPrecision> &a, char uplo = 'U'){
     int n = a.N();
@@ -1026,6 +1093,13 @@ class Linalg{
   };
   
 
+  static DenseVector<TPrecision> ExtractRow(Matrix<TPrecision> &a, int index){
+    DenseVector<TPrecision> v(a.N());
+    for(unsigned int i=0; i<a.N(); i++){
+      v(i) = a(index, i);
+    }
+    return v;
+  };
 
 
   static void ExtractRow(Matrix<TPrecision> &a, int index, Vector<TPrecision> &v){
@@ -1047,9 +1121,23 @@ class Linalg{
     for(unsigned int i=0; i<a.N(); i++){
       a(index, i) = v(i);
     }
+  };
+
+  
+  static void SetColumn(Matrix<TPrecision> &a, int index, TPrecision v){
+    for(unsigned int i=0; i<a.M(); i++){
+      a(i, index) = v;
+    }
+  };
+
+  static void SetRow(Matrix<TPrecision> &a, int index, TPrecision v){
+    for(unsigned int i=0; i<a.N(); i++){
+      a(index, i) = v;
+    }
   }; 
 
 
+
   static void SetColumn(Matrix<TPrecision> &a, int aindex,
       DenseMatrix<TPrecision> &b, int bindex){
     for(unsigned int i=0; i<a.M(); i++){
@@ -1092,6 +1180,15 @@ class Linalg{
         }
       }
   };
+
+  
+  static void SumColumn(Matrix<TPrecision> &a, int index){
+    double sum = 0;
+    for(unsigned int i=0; i < a.M(); i++){
+      sum += a(i, index);
+    }
+  };
+  
   
 
 
@@ -1138,13 +1235,19 @@ class Linalg{
 
 
   static void SubtractColumn(Matrix<TPrecision> &a, int index, Vector<TPrecision> &v,
-      Matrix<TPrecision> &out){
+      Matrix<TPrecision> &out)  {
       for(unsigned int i=0; i < a.M(); i++){
         out(i, index) = a(i, index) - v(i);
       }
   };
 
 
+  static void AddColumn(Matrix<TPrecision> &a, int index, Vector<TPrecision> &v,
+      Matrix<TPrecision> &out){
+      for(unsigned int i=0; i < a.M(); i++){
+        out(i, index) = a(i, index) + v(i);
+      }
+  };
 
   static void SubtractRowwise(Matrix<TPrecision> &a, Vector<TPrecision> &v,
       Matrix<TPrecision> &out){
@@ -1280,7 +1383,7 @@ class Linalg{
 
   static DenseVector<TPrecision> ColumnwiseNorm(Matrix<TPrecision> &A){
     DenseVector<TPrecision> norms(A.N());
-    ColumnwiseNOrm(A, norms);
+    ColumnwiseNorm(A, norms);
     return norms;
   };
 	  
@@ -1304,6 +1407,16 @@ class Linalg{
       result(i) = a(i) - b;
     }    
   };
+
+    //result = a - b
+  static void Subtract(Matrix<TPrecision> &a, TPrecision b, Matrix<TPrecision> &result){
+    for(unsigned int i = 0; i < a.M(); i++){
+      for(unsigned int j = 0; j < a.N(); j++){
+        result(i,j) = a(i, j) - b;
+      }
+    }    
+  };
+
   //result = a - b
   static void Add(Vector<TPrecision> &a, TPrecision b, Vector<TPrecision> &result){
     for(unsigned int i = 0; i < a.N(); i++){
@@ -1524,9 +1637,14 @@ class Linalg{
     }
   };   
   
-  //result = v*s
+  //returns 0 vector for vectors with 1/length = inf
   static void Normalize(Vector<TPrecision> &v){
-      Scale(v, 1.f/(Length(v)+1e-16), v);
+      TPrecision l = 1.0/Length(v);
+      if( l!=l || 
+          std::numeric_limits<TPrecision>::infinity() == l ){
+        l = 0;
+      }
+      Scale(v, l, v);
   }; 
   
 
@@ -1539,8 +1657,12 @@ class Linalg{
   };
 
   static void ScaleRow(DenseMatrix<TPrecision> &m, int index, TPrecision s){
+    ScaleRow(m, index, s, m);
+  };
+  
+  static void ScaleRow(DenseMatrix<TPrecision> &m, int index, TPrecision s, DenseMatrix<TPrecision> &out){
     for(unsigned int i=0; i < m.N(); i++){
-      m(index, i) = m(index, i) * s;
+      out(index, i) = m(index, i) * s;
     }
   };
 
@@ -1636,6 +1758,7 @@ class Linalg{
   };
 
 
+
   static bool IsColumnEqual(DenseMatrix<TPrecision> A, int ai, DenseMatrix<TPrecision>
       B, int bi){
     bool equal = true;
@@ -1646,6 +1769,7 @@ class Linalg{
   };
 
 
+
   static DenseVector<TPrecision> Max(Matrix<TPrecision> &A){
     DenseVector<TPrecision> m = Linalg<TPrecision>::ExtractColumn(A, 0);
     for(unsigned  int i=0;i<A.N(); i++){
@@ -1658,6 +1782,9 @@ class Linalg{
     return m;
   };
 
+
+
+
   static DenseVector<TPrecision> Min(Matrix<TPrecision> &A){
     DenseVector<TPrecision> m = Linalg<TPrecision>::ExtractColumn(A, 0);
     for(unsigned int i=0;i<A.N(); i++){
@@ -1692,6 +1819,25 @@ class Linalg{
   };
 
 
+  static void Print(DenseMatrix<TPrecision> &m){
+    for(int i=0; i<m.M(); i++){
+      for(int j=0; j< m.N(); j++){
+        std::cout << m(i, j) << ", ";
+      }
+      std::cout << std::endl;
+    }
+  };
+
+    static void Print(DenseVector<TPrecision> &v){
+      for(int j=0; j< v.N(); j++){
+        std::cout << v(j) << ", ";
+      }
+      std::cout << std::endl;
+  };
+
+
+
+
 };
 
 #endif
